local class = require('middleclass')
local Vector3 = class('Vector3')
function Vector3:initialize(x, y, z)
	if type(x) == "table" and x.class ~= nil and x.class.name == 'Vector3' then
		self.x = x.x
		self.y = x.y
		self.z = x.z
	else
		self.x = x or 0
		self.y = y or 0
		self.z = z or 0
	end
end
function Vector3:copy()
	return Vector3:new(self)
end
function Vector3:sub(v)
	self.x = self.x - v.x
	self.y = self.y - v.y
	self.z = self.z - v.z
	return self
end
function Vector3:cross(v)
	self.x = (self.y * v.z) - (self.z * v.y);
	self.y = (self.z * v.x) - (self.x * v.z);
	self.z = (self.x * v.y) - (self.y * v.x);
	return self
end
function Vector3:lengthSQ()
	return self.x*self.x + self.y*self.y + self.z*self.z
end
function Vector3:length()
	return math.sqrt(self:lengthSQ())
end
function Vector3:dot(v)
	return self.x * v.x
			+ self.y * v.y
			+ self.z * v.z;
end
function Vector3:normalize()
	local l = self:length()
	local scalar = 1/l
	self.x = self.x*scalar
	self.y = self.y*scalar
	self.z = self.z*scalar
	return self
end

return Vector3